/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "va_log.h"
#include "va_assert.h"
#include "varid.h"

/* NUC2COL translate nucleotide sequence to color-space.
 * [ res ] = NUC2COL(genome) translates a nucleotide sequence (ns or ls)
 * to color-space. Nucleotide can be in ns (0-4) or ls (a,c,g,t,-).
 *
 * Parameters:
 * innuc - the dna to be translated, in ns or ls.
 *
 * Returns:
 * outcol - the color-space sequence, oneless than
 *
 * See also COL2NUC, NUC2NUC, GET_COMPLEMENT
 *
 * Author: Adrian V. Dalca
 *
 *  TODO: fix bug when we get a genome starting with 4!
 */
VA_VecU8 nuc2col( 
    VA_VecU8 innuc)
{
    VA_VecU8 outcol;
    VA_VecU8 outcol_res;
    VA_VecU8 orig_genome;
    VA_VecU8 innuc2;
    int ischar = FALSE;
    int k = 0;
    int i = 0;

    for (i = 0; i < innuc.size; i++)
    {
       if (isalnum(innuc.data[i]) != 0)       
       {
           ischar = TRUE;
           break; 
       }
    }

    /* numbers to letters */
    if (ischar)
    {
        innuc2 = nuc2nuc(innuc);
    }
    else
    {
        innuc2 = va_allocVecU8(innuc.size);
        memcpy(innuc2.data, innuc.data, 
               innuc2.size * sizeof(VA_UINT8));
    }

    orig_genome = va_allocVecU8(innuc2.size);
    memcpy(orig_genome.data, innuc2.data, 
           orig_genome.size * sizeof(VA_UINT8));
    
    innuc2.size = 0;
    for (i = 0; i < orig_genome.size; i++)
    {
        if ((orig_genome.data[i] != 4) &&
            (orig_genome.data[i] != 5))
        {
            innuc2.data[innuc2.size++] = orig_genome.data[i];
        }
    } 

    /* get copies of the genome slightly off-frame. */
    /* build boolean vectors for each resulting color. */
    /* compute the final output */
    outcol = va_allocVecU8(orig_genome.size);
    memcpy(outcol.data, orig_genome.data, 
           outcol.size * sizeof(VA_UINT8));
    if (innuc2.size>0) {
    		VA_VecU8 outcol_tmp;
	    outcol_tmp = va_allocVecU8(innuc2.size - 1);
	    memset(outcol_tmp.data, (VA_UINT8)-1, 
		   outcol_tmp.size * sizeof(VA_UINT8));
	    
	    /* compute the no-gap output */
	    for (i = 0; i < innuc2.size - 1; i++)
	    {
		if (innuc2.data[i + 1] == innuc2.data[i])
		{
		    outcol_tmp.data[i] = 0;
		}
		else if (((innuc2.data[i + 1] == 0) &&
			  (innuc2.data[i] == 1)) ||
			 ((innuc2.data[i + 1] == 1) &&
			  (innuc2.data[i] == 0)) ||
			 ((innuc2.data[i + 1] == 2) &&
			  (innuc2.data[i] == 3)) ||
			 ((innuc2.data[i + 1] == 3) &&
			  (innuc2.data[i] == 2)))
		{
		    outcol_tmp.data[i] = 1;
		} 
		else if (((innuc2.data[i + 1] == 0) &&
			  (innuc2.data[i] == 2)) ||
			 ((innuc2.data[i + 1] == 2) &&
			  (innuc2.data[i] == 0)) ||
			 ((innuc2.data[i + 1] == 1) &&
			  (innuc2.data[i] == 3)) ||
			 ((innuc2.data[i + 1] == 3) &&
			  (innuc2.data[i] == 1)))
		{
		    outcol_tmp.data[i] = 2;
		}
		else if (((innuc2.data[i + 1] == 0) &&
			  (innuc2.data[i] == 3)) ||
			 ((innuc2.data[i + 1] == 3) &&
			  (innuc2.data[i] == 0)) ||
			 ((innuc2.data[i + 1] == 2) &&
			  (innuc2.data[i] == 1)) ||
			 ((innuc2.data[i + 1] == 1) &&
			  (innuc2.data[i] == 2)))
		{
		    outcol_tmp.data[i] = 3;
		} else {
			outcol_tmp.data[i]=4;
		}
		VA_ASSERT(outcol_tmp.data[i] != (VA_UINT8)-1);
	    }
 
	    k = -1;
	    for (i = 0; i < outcol.size; i++)
	    {
		if ((outcol.data[i] != 4) &&
		   (outcol.data[i] != 5))
		{
		    if (k == -1)
			outcol.data[i] = 0;
		    else
			outcol.data[i] = outcol_tmp.data[k];
		    k++;
		}
	    }
    		free(outcol_tmp.data);
    }
 
    outcol_res = va_allocVecU8(orig_genome.size - 1);
    memcpy(outcol_res.data, &outcol.data[1], 
           outcol_res.size * sizeof(VA_UINT8));
    free(outcol.data);
    free(innuc2.data);
    free(orig_genome.data);
    return (outcol_res);
}

